(*****************************************************************************)
(*                            DRAW2.INC                                      *)
(*                                                                           *)
(*               Unterprogramm-Modul zum Zeichnen der Funktion               *)
(*                   Speziell Schneider CPC 6128                             *)
(*****************************************************************************)


Procedure PlotFunction;


   Var xmin,xmax,
       ymin,ymax,
       dx,dy,x,d,
       s1,f1,f2   :Real;
       s,grad,i,j :Integer;
       key        :CHAR;
       ok         :BOOLEAN;

   (* Hier beginnt der hardwareabhaengige Teil *)

   Const ScreenXMax = 639;  (* Anzahl Bildpunkte horizontal                 *)
         ScreenYMax = 399;  (*                   vertikal                   *)
         Left       =  16;  (* Abstand Zeichenflaeche-linker Bildschirmrand *)
         Bottom     =  16;  (*                       -unterer               *)
         Unit       =   4;  (* halbe Breite der Achsenunterteilungsstriche  *)


   Procedure InitGraphic;     (* Voreinstellungen fuer Grafikbetrieb *)

      Begin
      ClrScr;
      Write (#27'0')
      End;


   Procedure LeaveGraphic;    (* Voreinstellungen fuer Textbetrieb *)

      Begin
      ClrScr;
      Write (#27'1')
      End;


   Procedure Plot (x,y :Integer);    (* Punkt bei (x,y) zeichnen *)

       Begin
       Inline ($2A/x/$EB/$2A/y/$CD/$10/$Fd/$EA/$BB)
       End;


   Procedure Draw (x1,y1,x2,y2 :Integer);  (* Linie von (x1,y1) nach (x2,y2) *)

      Begin
      Plot (x1,y1);
      Inline ($2A/x2/$EB/$2A/y2/$CD/$5A/$FC/$F6/$BB)
      End;

   (* hier endet der hardwareabhaengige Teil *)


  PROCEDURE worldtoscreen (x,y :REAL; VAR xs,ys :INTEGER);

  VAR a,b : REAL;

  BEGIN
    a := (x - xmin)*(screenxmax-left)/(xmax-xmin);
    b := (y - ymin)*(screenymax-bottom)/(ymax-ymin);
    IF Abs(a) > 32000.0
      THEN
        a := sign(a)*32000.0;
    IF Abs(b) > 32000.0
      THEN
        b := sign(b)*32000.0;
    xs := Round(a) + left;
    ys := Round(b) + bottom
  END;



   Procedure DrawPoint (x,y :Real);    (* zeichnet Punkt in Weltkoordinaten *)

      Var xp,yp :Integer;

      Begin
      WorldToScreen (x,y,xp,yp);
      Plot (xp,yp)
      End;


   Procedure DrawLine (x1,y1,x2,y2 :Real); (* zeichnet Linie in Weltkoordinaten *)

      Var xp1,yp1,xp2,yp2 :Integer;

      Begin
      WorldToScreen (x1,y1,xp1,yp1);
      WorldToScreen (x2,y2,xp2,yp2);
      Draw (xp1,yp1,xp2,yp2)
      End;


   Procedure DrawAxis;   (* zeichnet das Achsenkreuz *)

      Var x,y,Dist :Real;

      Begin
      Draw (Left,Bottom,Left,ScreenYMax);
      Draw (Left,Bottom,ScreenXMax,Bottom);
      DrawLine (xmin,0,xmax,0);
      DrawLine (0,ymin,0,ymax);

      dist := dy*(ScreenYMax-Bottom)/(ymax-ymin); (* Abstand y-Unterteilung *)
      x := Left;
      y := Bottom;
      While y<=ScreenYMax do
         Begin
         Draw (round(x-Unit),round(y),round(x+Unit),round(y));
         y := y + Dist
         End;

      dist := dx*(ScreenXMax-Left)/(xmax-xmin);   (* Abstand y-Unterteilung *)
      x := Left;
      y := Bottom;
      While x<=ScreenXMax do
         Begin
         Draw (round(x),round(y-Unit),round(x),round(y+Unit));
         x := x + Dist
         End;
      End;


BEGIN
  Clrscr;
  Writeln ('Funktion zeichnen');
  REPEAT
    Writeln;
    formulaln (xmin,'im Bereich von xmin = ');
    formulaln (xmax,'           bis xmax = ');
  UNTIL xmin < xmax;
  REPEAT
    formulaln (dx, 'mit Schrittweite dx = ');
  UNTIL dx > 0.0;
  REPEAT
    Writeln;
    formulaln (ymin,'           von ymin = ');
    formulaln (ymax,'           bis ymax = ');
  UNTIL ymin < ymax;
  REPEAT
    formulaln (dy, ' mit Schrittweite dy = ');
  UNTIL dy > 0.0;
  Writeln;
  REPEAT
    formulaln (s1,'Anzahl der zu berechnenden Stuetzstellen : ');
    s := Round(s1);
  UNTIL s >= 0;
  if s = 0 THEN s := screenxmax-left;
  Writeln;
  Writeln('Welche Funktion soll gezeichnet werden ?');
  Writeln;
  for i := 0 to maxgrad do
  BEGIN
    Write('         f');
    for j := 1 to i do Write('''');
    Write('':maxgrad-i, '  : ',i:3);
    WriteLn
  END;
  Writeln;
  Write('Bitte waehlen :   ');
  REPEAT
    Read(kbd,key);
  UNTIL key in ['0'..chr(maxgrad+ord('0'))];
  grad := ord(key) - ord('0');
  initgraphic;                               (* Grafik initialisieren *)
  drawaxis;                                  (* Achsenkreuz zeichnen  *)
  x := xmin;
  d := (xmax-xmin)/s;
  f1 := fn(x,grad);
  ok := calcresult;
  f2 := fn(x+d,grad);
  ok := ok AND calcresult;
  WHILE (x+d<=xmax) AND NOT Keypressed DO
    BEGIN
      IF ok
        THEN
          drawline (x,f1,x+d,f2);     (* Zeichnen der Funktion *)
      x := x + d;
      f1 := fn(x,grad);
      ok := calcresult;
      f2 := fn(x+d,grad);
      ok := ok AND calcresult;
    END;
  REPEAT UNTIL keypressed;
  leavegraphic
END;

